"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.success = exports.print = exports.test = void 0;
// Generated by CoffeeScript 2.3.1
var defaults, print, race, success, test, timeout, timer;
exports.test = test;
exports.success = success;
exports.print = print;

require("colors");

timer = function (t) {
  return new Promise(function (_, nope) {
    return setTimeout(function () {
      return nope(new Error("Test timed out"));
    }, t);
  });
};

race = function (...promises) {
  return Promise.race([...promises]);
};

timeout = function (t, ...promises) {
  return race(timer(t), ...promises);
};

defaults = {
  wait: 500
};
exports.success = success = true; // TODO: use explicit result objects, instead of true | Error | undefined

exports.test = test = async function (description, definition) {
  var error, result, wait;

  if (description.constructor === Object) {
    ({
      description,
      wait
    } = description);
  } else if (description.constructor === String) {
    ({
      wait
    } = defaults);
  } else {
    description = description.toString();
  }

  if (definition != null) {
    if (Array.isArray(definition)) {
      return [// TODO: include error/timeout/pending count in result object
      description, await async function () {
        var i, len, results;
        results = [];

        for (i = 0, len = definition.length; i < len; i++) {
          result = definition[i];
          results.push((await result));
        }

        return results;
      }()];
    } else if (definition.call != null) {
      try {
        if (wait === false) {
          await definition();
        } else {
          await timeout(wait, definition());
        }

        return [description, true];
      } catch (error1) {
        error = error1;
        exports.success = success = false; // at least one failing test

        if (error.message !== "Test timed out") {
          console.error(`${error.stack.red}`);
        }

        return [description, error];
      }
    } else {
      return [description, new Error("Invalid test definition")];
    }
  } else {
    return [description, void 0];
  }
}; // TODO: add error counts for groups
// TODO: groups with failing/pending tests should be red


exports.print = print = function ([description, result], indent = "") {
  var i, len, r, results;

  if (Array.isArray(result)) {
    console.error(indent, description.blue);
    results = [];

    for (i = 0, len = result.length; i < len; i++) {
      r = result[i];
      results.push(print(r, indent + "  "));
    }

    return results;
  } else {
    return console.error(indent, result != null ? result === true ? description.green : result.message != null && result.message !== "" ? `${description.red} (${result.message.red})` : description.red : description.yellow);
  }
};